@@ -1,8 +1,9 @@
 /**
  * 
- * $Header: /cvsroot/hungry/lesstif/include/Motif-2.0/Xm/SpinB.h,v 1.3 1997/08/25 13:05:09 u27113 Exp $
+ * $Header: /cvsroot/lesstif/lesstif/include/Motif-2.1/Xm/SpinB.h,v 1.1 2004/08/28 19:23:26 dannybackx Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
+ * Copyright (C) 1997-2000 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,8 +23,8 @@
  *
  **/
 
-#ifndef XM_SPINB_H
-#define XM_SPINB_H
+#ifndef _XM_SPINB_H
+#define _XM_SPINB_H
 
 #include <Xm/Xm.h>
 
@@ -31,7 +32,7 @@
 extern "C" {
 #endif
 
-extern WidgetClass xmSpinBoxWidgetClass;
+XMLIBEXPORT extern WidgetClass xmSpinBoxWidgetClass;
 
 typedef struct _XmSpinBoxClassRec *XmSpinBoxWidgetClass;
 typedef struct _XmSpinBoxRec *XmSpinBoxWidget;
@@ -40,11 +41,14 @@ typedef struct _XmSpinBoxRec *XmSpinBoxWidget;
 #define XmIsSpinBox(w) XtIsSubclass((w), xmSpinBoxWidgetClass)
 #endif
 
-extern Widget XmCreateSpinBox(Widget parent, char *name, ArgList arglist, Cardinal argcount);
+XMLIBEXPORT Widget XmCreateSpinBox(Widget parent, char *name, ArgList arglist,
+                       Cardinal argcount);
+
+XMLIBEXPORT int XmSpinBoxValidatePosition(Widget textfield,
+                              int *position);
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* XM_SPINB_H */
-
+#endif /* _XM_SPINB_H */
