@@ -1,9 +1,9 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm-2.1/TextOut.c,v 1.7 2005/03/29 15:28:00 dannybackx Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm-2.1/TextOut.c,v 1.9 2006/04/19 18:42:22 dannybackx Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
- * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 LessTif Development Team
+ * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm-2.1/TextOut.c,v 1.7 2005/03/29 15:28:00 dannybackx Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm-2.1/TextOut.c,v 1.9 2006/04/19 18:42:22 dannybackx Exp $";
 
 /* Do NOT use this (USE_XT_GC) :
  * it requires setting all the dynamic GC fields before every use of
@@ -209,7 +209,7 @@ FontInit(XmTextWidget w)
 	int i;
 
 	DEBUGOUT(_LtDebug(__FILE__, (Widget)w, "FontInit(fontlist %p)\n", oldlist));
-	DEBUGOUT(_LtDebugPrintRenderTable("FontInit(): ", oldlist));
+	DEBUGOUT(_LtDebugPrintRenderTable(__FILE__, (Widget)w, "FontInit(): ", oldlist));
 
 	/* This part is probably all wrong ... FIX ME */
 	for (i = 0; Out_FontList(o)->renditions[i]->tag != NULL; i++) {
@@ -366,7 +366,14 @@ FontInit(XmTextWidget w)
 		Out_FontAverageWidth(o) = _XmFontCalculateAverageCharacterWidth((Widget)w, fs);
 	}
 	Out_TabWidth(o) = 8 * Out_FontAverageWidth(o);
+#ifdef	USE_XFT
+	if (Out_XftFont(o))
+		Out_XftFont(o)->height = Out_Font_Ascent(o) + Out_Font_Descent(o);
+	else
+		o->lineheight = Out_Font_Ascent(o) + Out_Font_Descent(o);
+#else
 	Out_FontHeight(o) = Out_Font_Ascent(o) + Out_Font_Descent(o);
+#endif
 	Out_CursorHeight(o) = Out_FontHeight(o);
 
 
@@ -934,7 +941,7 @@ _XmTextNextX(XmTextWidget w, int x, char *ptr, int len)
             } else if (Out_XftFont(o)) {
 	    	XGlyphInfo	ext;
 		XftTextExtents8(XtDisplay((Widget)w), Out_XftFont(o),
-			ptr, 1, &ext);
+			(unsigned char*)ptr, 1, &ext);
 		x += ext.width;
 #endif
 	    } else {
@@ -3340,7 +3347,7 @@ _XmOut_FontTextWidth(OutputData o, char *s, int l)
 	} else if (Out_XftFont(o)) {
 		XGlyphInfo	ext;
 		XftTextExtents8(Out_FontList(o)->renditions[0]->dpy,
-			Out_XftFont(o), s, l, &ext);
+			Out_XftFont(o), (unsigned char*)s, l, &ext);
 
 		return ext.width;
 #endif
