@@ -3,7 +3,6 @@
  */
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdarg.h>
 
 #include <Xm/BulletinB.h>
 #include <Xm/SpinB.h>
@@ -12,6 +11,9 @@
 #include <Xm/TraitP.h>
 #include <Xm/AccTextT.h>
 
+#include "../../common/Test.h"
+
+
 Widget		toplevel, spb, bb, day, month, year;
 
 void ModifyVerify(Widget w, XtPointer client, XtPointer call)
