@@ -1,4 +1,8 @@
+/* $Header: /cvsroot/lesstif/lesstif/test/Xm/form/test14.c,v 1.6 2001/05/16 09:19:46 amai Exp $ */
+
 #include <stdio.h>
+#include <stdlib.h>
+
 #include <Xm/Xm.h>
 #include <Xm/PushB.h>
 #include <Xm/Form.h>
@@ -7,7 +11,7 @@
 #include <Xm/TextF.h>
 #include <Xm/SeparatoG.h>
 
-#include "DumpLayout.h"
+#include "../../common/Test.h"
 
 static char *FallBack[] = {
 		"*.borderWidth: 1",
