@@ -1,4 +1,4 @@
-/* $Id: test15.c,v 1.6 2000/09/02 13:40:43 amai Exp $ */
+/* $Id: test15.c,v 1.7 2001/05/30 08:15:14 amai Exp $ */
 /** --------------------------------------------------------------- **/
 
 #include <stdlib.h>
@@ -153,7 +153,7 @@ void InitTextInternals ( )
 	memset ( &copyOfTextInputInternals, 170, sizeof ( InputDataRec ) );
 	memset ( &copyOfTextOutputInternals, 170, sizeof ( OutputDataRec ) );
 	memset ( &copyOfRangeAtIndex0, 170, sizeof ( RangeRec ) );
-	memset ( &copyOfLineRec, 170, sizeof ( LineRec ) * 100 ) ; 
+	memset ( copyOfLineRec, 170, sizeof ( LineRec ) * 100 ) ; 
 /*	memset ( &copyOfLineTable, 170, sizeof ( LineTable ) * 100 ) ; 
 */
 /*CP: due to a change in type from Motif 1.2 to 2.0 */
@@ -169,10 +169,10 @@ void CopyTextInternals ( XmTextWidget toBeCopied )
 	memcpy ( &copyOfTextInputInternals, toBeCopied -> text.input->data, sizeof ( InputDataRec ) );
 	memcpy ( &copyOfTextOutputInternals, toBeCopied -> text.output->data, sizeof ( OutputDataRec ) );
 	memcpy ( &copyOfRangeAtIndex0, &toBeCopied -> text.repaint.range[0], sizeof ( RangeRec ) );
-	memcpy ( &copyOfLineRec, toBeCopied -> text.line, 
+	memcpy ( copyOfLineRec, toBeCopied -> text.line, 
 		sizeof ( LineRec ) * ( toBeCopied -> text.maximum_lines < 100 ? toBeCopied -> text.maximum_lines : 100 ) );
 	if ( toBeCopied -> text.line_table ) 
-		memcpy ( &copyOfLineTable, toBeCopied -> text.line_table, 
+		memcpy ( copyOfLineTable, toBeCopied -> text.line_table, 
 			sizeof ( XmTextLineTableRec ) * ( toBeCopied -> text.total_lines < 100 ? toBeCopied -> text.total_lines : 100 ) );
 }
 
