@@ -1,6 +1,11 @@
+/* $Header: /cvsroot/lesstif/lesstif/test/Xm/xmstring/test3.c,v 1.3 2002/04/17 16:32:01 amai Exp $ */
+
+#include <stdlib.h>
+#include <stdio.h>
+
 #include <Xm/XmP.h>
 #include <Xm/DrawingA.h>
-#include <stdio.h>
+
 
 GC gc;
 XmFontList fl;
@@ -64,6 +69,13 @@ main(int argc,
 
     xmstr = XmStringCreate("Hello World", XmFONTLIST_DEFAULT_TAG);
 
-    XtAppMainLoop(app);
+    
+{
+    static XtWidgetGeometry Expected[] = {
+   CWWidth | CWHeight            ,   56,   72,    1,    1, 0,0,0, /* da */ 
+    };
+    PrintDetails(    toplevel ,Expected);
+};
+   LessTifTestMainLoop(    toplevel );
     exit(0);
 }
