@@ -1,8 +1,9 @@
 /**
  *
- * $Id: Test.c,v 1.22 1999/10/23 12:39:29 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/test/common/Test.c,v 1.37 2002/05/01 15:19:15 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
+ * Copyright (C) 1996-2002 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,21 +23,32 @@
  *
  **/
 
+#include "LTTconfig.h"
+ 
 #include <stdlib.h>
 #include <stdio.h>
-#include <X11/X.h>
+#include <string.h>
+
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/keysym.h>
 #include <X11/Core.h>
 
+#define LIB_LTTEST
+#include "Test.h"
+
 /* ************** */
 
 int GlobalErrors = 0;
 
-static Time SynthTime = 0;
-
+/* local prototypes */
+static void Initialize(Widget w);
 static void Delay(Widget w);
+static void Exit(Widget w);
+static void Popup(Widget w, Boolean *mapped, XEvent *event);
+static void TimeOut(Boolean *mapped);
+static void LessTifTestPrintEvent(Widget w, XEvent *event);
+static void LessTifTestProcessEvent(Widget w);
 
 typedef struct
 {
@@ -45,9 +57,7 @@ typedef struct
     int geometrySlop;
     Boolean printEvents;
     String dumpOnFailureFile;
-}
-AppResources_t, *AppResourcesPtr;
-
+} AppResources_t, *AppResourcesPtr;
 static AppResources_t AppResources;
 
 static XtResource resources[] =
@@ -61,7 +71,7 @@ static XtResource resources[] =
 
 /* ************** */
 
-char *
+extern const char *
 XdbGeometryResult2String(XtGeometryResult r)
 {
     switch (r)
@@ -85,7 +95,7 @@ XdbGeometryResult2String(XtGeometryResult r)
 
 /* ************** */
 
-char *
+extern const char *
 XdbWidgetGeometry2String(XtWidgetGeometry *g)
 {
     static char o1[128], o2[128], b[20], *out = NULL;
@@ -150,16 +160,31 @@ XdbWidgetGeometry2String(XtWidgetGeometry *g)
     return out;
 }
 
+
+extern const char *
+XdbBoolean2String(int b)
+{
+    if (b)
+    {
+	return "True";
+    } else
+    {
+	return "False";
+    }
+}
+
+
 /* ************** */
 
 static void
 Initialize(Widget w)
 {
     static Boolean inited = False;
-    Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
 
     if (!inited)
     {
+      Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
+
 	XtGetApplicationResources(w,
 				  &AppResources,
 				  resources, XtNumber(resources),
@@ -171,7 +196,7 @@ Initialize(Widget w)
 
 /* ************** */
 
-int
+extern int
 LessTifTestGetSlop(Widget w)
 {
     Initialize(w);
@@ -180,7 +205,7 @@ LessTifTestGetSlop(Widget w)
 
 /* ************** */
 
-void
+extern void
 LessTifTestSetSlop(Widget w, int slop)
 {
     Initialize(w);
@@ -192,9 +217,9 @@ LessTifTestSetSlop(Widget w, int slop)
 static void
 Exit(Widget w)
 {
-    Display *display = 
+/*    Display *display = 
       XtIsSubclass(w,coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
-    XtAppContext app_context=XtWidgetToApplicationContext(w);
+    XtAppContext app_context=XtWidgetToApplicationContext(w); */
 
     /* destroy application and reclaim memory */ 
 
@@ -225,13 +250,13 @@ LessTifTestPrintEvent(Widget w, XEvent *event)
 	    fprintf(stdout, "KeyRelease\n");
 	    break;
 	case ButtonPress:
-	    fprintf(stdout, "ButtonPress %s %d\n",
+	    fprintf(stdout, "ButtonPress %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
 	    break;
 	case ButtonRelease:
-	    fprintf(stdout, "ButtonRelease %s %d\n",
+	    fprintf(stdout, "ButtonRelease %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
@@ -240,25 +265,25 @@ LessTifTestPrintEvent(Widget w, XEvent *event)
 	    fprintf(stdout, "MotionNotify\n");
 	    break;
 	case EnterNotify:
-	    fprintf(stdout, "EnterNotify %s %d\n",
+	    fprintf(stdout, "EnterNotify %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
 	    break;
 	case LeaveNotify:
-	    fprintf(stdout, "LeaveNotify %s %d\n",
+	    fprintf(stdout, "LeaveNotify %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
 	    break;
 	case FocusIn:
-	    fprintf(stdout, "FocusIn %s %d\n",
+	    fprintf(stdout, "FocusIn %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
 	    break;
 	case FocusOut:
-	    fprintf(stdout, "FocusOut %s %d\n",
+	    fprintf(stdout, "FocusOut %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
@@ -267,7 +292,7 @@ LessTifTestPrintEvent(Widget w, XEvent *event)
 	    fprintf(stdout, "KeymapNotify\n");
 	    break;
 	case Expose:
-	    fprintf(stdout, "Expose %s %d\n",
+	    fprintf(stdout, "Expose %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
@@ -288,13 +313,13 @@ LessTifTestPrintEvent(Widget w, XEvent *event)
 	    fprintf(stdout, "DestroyNotify\n");
 	    break;
 	case UnmapNotify:
-	    fprintf(stdout, "UnmapNotify %s %d\n",
+	    fprintf(stdout, "UnmapNotify %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
 	    break;
 	case MapNotify:
-	    fprintf(stdout, "MapNotify %s %d\n",
+	    fprintf(stdout, "MapNotify %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
@@ -306,7 +331,7 @@ LessTifTestPrintEvent(Widget w, XEvent *event)
 	    fprintf(stdout, "ReparentNotify\n");
 	    break;
 	case ConfigureNotify:
-	    fprintf(stdout, "ConfigureNotify %s %d\n",
+	    fprintf(stdout, "ConfigureNotify %s %ld\n",
 	    	XtName(XtWindowToWidget(display, event->xany.window)),
 	    	event->xany.serial
 	    	);
@@ -350,8 +375,9 @@ LessTifTestPrintEvent(Widget w, XEvent *event)
 	case LASTEvent:
 	    fprintf(stdout, "LASTEvent\n");
 	    break;
+
 	default:
-	    fprintf(stdout, "event\n");
+	    fprintf(stdout, "Unknown event\n");
 	    break;
 	}
 }
@@ -361,7 +387,7 @@ LessTifTestPrintEvent(Widget w, XEvent *event)
 static void
 LessTifTestProcessEvent(Widget w)
 {
-XEvent event;
+	XEvent event;
 
 	if (XtIMXEvent == XtAppPending(XtWidgetToApplicationContext(w)))
 	{
@@ -408,14 +434,14 @@ TimeOut(Boolean *mapped)
 
 /* ************** */
 
-void
+extern void
 LessTifTestDelay(Widget w, unsigned long interval)
 {
     Boolean mapped = False;
     XtAppContext app_context=XtWidgetToApplicationContext(w);
 
-    XtAppAddTimeOut(app_context, interval, (void *)TimeOut,
-		    (XtPointer)&mapped); 
+    XtAppAddTimeOut(app_context, interval,
+                    (XtTimerCallbackProc)TimeOut, (XtPointer)&mapped); 
     while (!mapped)
     {
 	XtAppProcessEvent(app_context, XtIMTimer);
@@ -432,7 +458,7 @@ Delay(Widget w)
 
 /* ************** */
 
-void
+extern void
 LessTifTestFlushEvents(Widget w)
 {
     Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
@@ -448,7 +474,8 @@ LessTifTestFlushEvents(Widget w)
 
 /* ************** */
 
-void LessTifTestPrintEvents(Widget w, Boolean flag)
+extern void
+LessTifTestPrintEvents(Widget w, Boolean flag)
 {
 	LessTifTestFlushEvents(w);
 	AppResources.printEvents = flag;
@@ -456,7 +483,7 @@ void LessTifTestPrintEvents(Widget w, Boolean flag)
 
 /* ************** */
 
-int
+extern int
 LessTifTestResizeWidget(Widget w, Dimension wt, Dimension ht)
 {
     Dimension bw;
@@ -472,7 +499,7 @@ LessTifTestResizeWidget(Widget w, Dimension wt, Dimension ht)
 
 /* ************** */
 
-int
+extern int
 LessTifTestWaitForIt(Widget w)
 {
     XWindowAttributes window_attributes;
@@ -485,14 +512,16 @@ LessTifTestWaitForIt(Widget w)
     if (window_attributes.map_state == IsUnmapped)
     {
 
-	XtAddEventHandler(w, StructureNotifyMask, False, (void *)Popup, (XtPointer)&mapped);
+	XtAddEventHandler(w, StructureNotifyMask, False,
+	                  (XtEventHandler)Popup, (XtPointer)&mapped);
 	XSync(display, False);
 	while (!mapped || XtAppPending(XtWidgetToApplicationContext(w)))
 	{
 	    XtAppProcessEvent(XtWidgetToApplicationContext(w), XtIMAll);
 	    XFlush(display);
 	}
-	XtRemoveEventHandler(w, StructureNotifyMask, False, (void *)Popup, (XtPointer)&mapped);
+	XtRemoveEventHandler(w, StructureNotifyMask, False,
+	                     (XtEventHandler)Popup, (XtPointer)&mapped);
     }
     Delay(w);
     return (0);
@@ -500,7 +529,7 @@ LessTifTestWaitForIt(Widget w)
 
 /* ************** */
 
-int
+extern int
 LessTifTestMainLoop(Widget w)
 {
     Initialize(w);
@@ -529,7 +558,7 @@ LessTifTestMainLoop(Widget w)
 
 /* ************** */
 
-void
+extern void
 LessTifTestWarpPointerAbove(Widget w)
 {
     Dimension width, height;
@@ -554,7 +583,7 @@ LessTifTestWarpPointerAbove(Widget w)
 
 /* ************** */
 
-void
+extern void
 LessTifTestWarpPointer(Widget w)
 {
     Dimension width, height;
@@ -579,7 +608,99 @@ LessTifTestWarpPointer(Widget w)
 
 /* ************** */
 
-void
+extern void
+LessTifTestKeyRelease(Widget w, KeySym keysym, unsigned int state)
+{
+    Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
+    Window window = XtIsSubclass(w, coreWidgetClass) ? XtWindow(w) : XtWindow(XtParent(w));
+    XKeyEvent key_event =
+    {
+	KeyRelease,
+	0,
+	False,
+	NULL, /* display, */
+	None, /* window, */
+	None, /* root, */
+	None, /* subwindow, */
+	CurrentTime,
+	0, 0,
+	0, 0,
+	0,
+	0x09,
+	True
+    };
+    XEvent *event;
+    unsigned int keys_buttons;
+
+    key_event.keycode = XKeysymToKeycode(display, keysym);
+    key_event.display=display;
+    key_event.state=state;
+    key_event.window=key_event.root=key_event.subwindow=window;
+    LessTifTestWarpPointer(w);
+    XQueryPointer(display, window,
+		  &key_event.root,
+		  &key_event.subwindow,
+		  &key_event.x_root, &key_event.y_root,
+		  &key_event.x, &key_event.y,
+		  &keys_buttons);
+    event = (XEvent*)&key_event;
+    event->xany.serial = NextRequest(display);
+    event->xbutton.time = XtLastTimestampProcessed(display);
+    LessTifTestDelay(w, 200);
+    XSendEvent(display, key_event.window, True, KeyReleaseMask, event);
+    LessTifTestFlushEvents(w);
+    Delay(w);
+}
+
+/* ************** */
+
+extern void
+LessTifTestKeyPress(Widget w, KeySym keysym, unsigned int state)
+{
+    Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
+    Window window = XtIsSubclass(w, coreWidgetClass) ? XtWindow(w) : XtWindow(XtParent(w));
+    XKeyEvent key_event =
+    {
+	KeyPress,
+	0,
+	False,
+	NULL, /* display, */
+	None, /* window, */
+	None, /* root, */
+	None, /* subwindow, */
+	CurrentTime,
+	0, 0,
+	0, 0,
+	0,
+	0x09,
+	True
+    };
+    XEvent *event;
+    unsigned int keys_buttons;
+
+    key_event.keycode = XKeysymToKeycode(display, keysym);
+    key_event.display=display;
+    key_event.state=state;
+    key_event.window=key_event.root=key_event.subwindow=window;
+    LessTifTestWarpPointer(w);
+    XQueryPointer(display, window,
+		  &key_event.root,
+		  &key_event.subwindow,
+		  &key_event.x_root, &key_event.y_root,
+		  &key_event.x, &key_event.y,
+		  &keys_buttons);
+    event = (XEvent*)&key_event;
+    event->xany.serial = NextRequest(display);
+    event->xbutton.time = XtLastTimestampProcessed(display);
+    LessTifTestDelay(w, 200);
+    XSendEvent(display, InputFocus, True, KeyPressMask, event);
+    LessTifTestFlushEvents(w);
+    Delay(w);
+}
+
+/* ************** */
+
+extern void
 LessTifTestKeysym(Widget w, KeySym keysym)
 {
     Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
@@ -615,18 +736,19 @@ LessTifTestKeysym(Widget w, KeySym keysym)
 		  &keys_buttons);
     event = (XEvent*)&key_event;
     event->xany.serial = NextRequest(display);
-    event->xbutton.time = SynthTime;
-    SynthTime += 200;
-    XSendEvent(display, PointerWindow, True, KeyPressMask, event);
+    event->xbutton.time = XtLastTimestampProcessed(display);
+    LessTifTestDelay(w, 200);
+    XSendEvent(display, InputFocus, True, KeyPressMask, event);
     key_event.type = KeyRelease;
-    XSendEvent(display, PointerWindow, True, KeyReleaseMask, event);
+    key_event.window = XtWindow(XtParent(w));
+    XSendEvent(display, key_event.window, True, KeyReleaseMask, event);
     LessTifTestFlushEvents(w);
     Delay(w);
 }
 
 /* ************** */
 
-void
+extern void
 LessTifTestEsc(Widget w)
 {
     LessTifTestKeysym(w, XK_Escape);
@@ -634,8 +756,8 @@ LessTifTestEsc(Widget w)
 
 /* ************** */
 
-void
-LessTifTestBtn1Down(Widget w)
+extern void
+LessTifTestBtnDown(Widget w, int button)
 {
     Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
     Window window = XtIsSubclass(w, coreWidgetClass) ? XtWindow(w) : XtWindow(XtParent(w));
@@ -658,6 +780,7 @@ LessTifTestBtn1Down(Widget w)
     XEvent *event;
     unsigned int keys_buttons;
 
+    button_event.button = button;
     button_event.display=display;
     button_event.window=button_event.root=button_event.subwindow=window;
     LessTifTestWarpPointer(w);
@@ -669,8 +792,8 @@ LessTifTestBtn1Down(Widget w)
 		  &keys_buttons);
     event = (XEvent*)&button_event;
     event->xany.serial = NextRequest(display);
-    event->xbutton.time = SynthTime;
-    SynthTime += 200;
+    event->xbutton.time = XtLastTimestampProcessed(display);
+    LessTifTestDelay(w, 200);
     XSendEvent(display, PointerWindow, True, ButtonPressMask, event);
     LessTifTestFlushEvents(w);
     Delay(w);
@@ -678,8 +801,8 @@ LessTifTestBtn1Down(Widget w)
 
 /* ************** */
 
-void
-LessTifTestBtn1Up(Widget w)
+extern void
+LessTifTestBtnUp(Widget w, int button)
 {
     Display *display = XtIsSubclass(w, coreWidgetClass) ? XtDisplay(w) : XtDisplay(XtParent(w));
     Window window = XtIsSubclass(w, coreWidgetClass) ? XtWindow(w) : XtWindow(XtParent(w));
@@ -702,6 +825,22 @@ LessTifTestBtn1Up(Widget w)
     XEvent *event;
     unsigned int keys_buttons;
 
+    switch (button)
+    {
+    case 1:
+	button_event.state = Button1Mask;
+	break;
+    case 2:
+	button_event.state = Button2Mask;
+	break;
+    case 3:
+	button_event.state = Button3Mask;
+	break;
+    default:
+	button_event.state = 0;
+    	break;
+    }
+    button_event.button = button;
     button_event.display=display;
     button_event.window=button_event.root=button_event.subwindow=window;
     LessTifTestWarpPointer(w);
@@ -713,8 +852,8 @@ LessTifTestBtn1Up(Widget w)
 		  &keys_buttons);
     event = (XEvent*)&button_event;
     event->xany.serial = NextRequest(display);
-    event->xbutton.time = SynthTime;
-    SynthTime += 200;
+    event->xbutton.time = XtLastTimestampProcessed(display);
+    LessTifTestDelay(w, 200);
     XSendEvent(display, PointerWindow, True, ButtonReleaseMask, event);
     LessTifTestFlushEvents(w);
     Delay(w);
@@ -722,7 +861,55 @@ LessTifTestBtn1Up(Widget w)
 
 /* ************** */
 
-int
+extern void
+LessTifTestBtn1Down(Widget w)
+{
+    LessTifTestBtnDown(w, 1);
+}
+
+/* ************** */
+
+extern void
+LessTifTestBtn1Up(Widget w)
+{
+    LessTifTestBtnUp(w, 1);
+}
+
+/* ************** */
+
+extern void
+LessTifTestBtn2Down(Widget w)
+{
+    LessTifTestBtnDown(w, 2);
+}
+
+/* ************** */
+
+extern void
+LessTifTestBtn2Up(Widget w)
+{
+    LessTifTestBtnUp(w, 2);
+}
+
+/* ************** */
+
+extern void
+LessTifTestBtn3Down(Widget w)
+{
+    LessTifTestBtnDown(w, 3);
+}
+
+/* ************** */
+
+extern void
+LessTifTestBtn3Up(Widget w)
+{
+    LessTifTestBtnUp(w, 3);
+}
+
+/* ************** */
+
+extern int
 LessTifTestPushButton(Widget w)
 {
     Initialize(w);
